@@ -49,8 +49,7 @@ public:
         const string& query) const;
 };
 
-StrigiHtmlGui::StrigiHtmlGui(HtmlHelper* h) : helper(h) {
-    p = new Private(helper);
+StrigiHtmlGui::StrigiHtmlGui(HtmlHelper* h) :helper(h), p(new Private(helper)) {
 }
 StrigiHtmlGui::~StrigiHtmlGui() {
     delete p;
@@ -113,7 +112,8 @@ exists(const char* file) {
 }
 void
 startDaemon() {
-    char* const daemon = "strigidaemon";
+    char daemon[13];
+    strcpy(daemon, "strigidaemon");
     // find the executable
     // get the PATH environment
     const char* path = getenv("PATH");
@@ -127,8 +127,8 @@ startDaemon() {
         end = strchr(path, ':');
         if (exists(p.c_str())) {
             if (fork()) {
-                char * const args[] = {daemon, "clucene", 0};
-                execvp(p.c_str(), args);
+                const char * const args[] = {daemon, "clucene", 0};
+                execvp(p.c_str(), (char **)args);
             }
             break;
         }
