@@ -29,6 +29,7 @@
 #include "cluceneindexreader.h"
 #include "cluceneindexmanager.h"
 #include <CLucene/util/stringreader.h>
+#include <CLucene/search/PrefixQuery.h>
 #include <sstream>
 #include <assert.h>
 
@@ -43,8 +44,12 @@ using lucene::document::Field;
 using lucene::index::IndexWriter;
 using lucene::index::Term;
 using lucene::index::TermDocs;
+using lucene::search::BooleanQuery;
 using lucene::search::IndexSearcher;
 using lucene::search::Hits;
+using lucene::search::PrefixFilter;
+using lucene::search::TermQuery;
+using lucene::search::Query;
 using lucene::util::BitSet;
 
 using lucene::util::Reader;
@@ -69,6 +74,13 @@ CLuceneIndexWriter::systemlocation() {
     const static wstring s(utf8toucs2(FieldRegister::pathFieldName));
     return s.c_str();
 }
+namespace {
+const wchar_t*
+parentlocation() {
+    const static wstring s(utf8toucs2(FieldRegister::parentLocationFieldName));
+    return s.c_str();
+}
+}
 void
 CLuceneIndexWriter::addText(const AnalysisResult* idx, const char* text,
         int32_t length) {
@@ -103,18 +115,23 @@ CLuceneIndexWriter::addValue(const AnalysisResult* idx,
         const TCHAR* value) {
     CLuceneDocData* doc = static_cast<CLuceneDocData*>(idx->writerData());
     int config = 0;
-    if ( (type & AnalyzerConfiguration::Stored) == AnalyzerConfiguration::Stored )
-	config |= Field::STORE_YES;
-    else
-	config |= Field::STORE_NO;
+    if ((type & AnalyzerConfiguration::Stored) == AnalyzerConfiguration::Stored){
+        config |= Field::STORE_YES;
+    } else {
+        config |= Field::STORE_NO;
+    }
 
-    if ( (type & AnalyzerConfiguration::Indexed) == AnalyzerConfiguration::Indexed ){
-	    if ( (type & AnalyzerConfiguration::Tokenized) == AnalyzerConfiguration::Tokenized )
-		config |= Field::INDEX_TOKENIZED;
-	    else
-		config |= Field::INDEX_UNTOKENIZED;
-    }else
-	config |= Field::INDEX_NO;
+    if ((type & AnalyzerConfiguration::Indexed)
+            == AnalyzerConfiguration::Indexed) {
+        if ((type & AnalyzerConfiguration::Tokenized)
+                == AnalyzerConfiguration::Tokenized) {
+            config |= Field::INDEX_TOKENIZED;
+        } else {
+            config |= Field::INDEX_UNTOKENIZED;
+        }
+    } else {
+        config |= Field::INDEX_NO;
+    }
 
     Field* field = new Field(name, value, config);
     doc->doc.add(*field);
@@ -240,26 +257,38 @@ CLuceneIndexWriter::deleteEntries(const std::vector<std::string>& entries) {
 void
 CLuceneIndexWriter::deleteEntry(const string& entry,
         lucene::index::IndexReader* reader) {
+    int deleted = 0;
+    wstring path(utf8toucs2(entry));
+{
+    Term t(systemlocation(), path.c_str());
+    deleted += reader->deleteDocuments(&t);
+    // if no file was deleted, no more can be deleted
+    if (deleted == 0) return;
+}
+{
+    Term t(parentlocation(), path.c_str());
+    deleted += reader->deleteDocuments(&t);
 
-    wstring tstr(utf8toucs2(entry));
-    int32_t prefixLen = tstr.length();
-    const TCHAR* prefixText = tstr.c_str();
-    int32_t maxdoc = reader->maxDoc();
-    for (int32_t i = 0; i < maxdoc; ++i) {
-        if (!reader->isDeleted(i)) {
-            Document* d = reader->document(i);
-            const TCHAR* t = d->get(systemlocation());
-            if (t && wcsncmp(t, prefixText, prefixLen) == 0) {
-                try {
-                    reader->deleteDocument(i);
-                } catch (CLuceneError& err) {
-                    cerr << "Could not delete document '" << entry
-                        << "' from the index: " << err.what() << endl;
-                }
-            }
-            _CLDELETE(d);
+    // if we have only deleted one file up to now, we cannot delete more
+    if (deleted < 2) return;
+}
+{
+    // delete all deeper nested files
+    wstring v = utf8toucs2(entry+"/");
+    Term* t = _CLNEW Term(parentlocation(), v.c_str());
+    PrefixFilter* filter = _CLNEW PrefixFilter(t);
+    BitSet* b = filter->bits(reader);
+    _CLDELETE(filter);
+    _CLDECDELETE(t);
+    int32_t size = b->size();
+    for (int id = 0; id < size; ++id) {
+        if (b->get(id) && !reader->isDeleted(id)) {
+            reader->deleteDocument(id);
+            deleted++;
         }
     }
+    _CLDELETE(b);
+}
 }
 void
 CLuceneIndexWriter::deleteAllEntries() {
@@ -348,7 +377,7 @@ CLuceneIndexWriter::cleanUp() {
         return;
     }
     /*
-	//this is a hack and will not work with new versions of the index..
+    //this is a hack and will not work with new versions of the index..
         //furthermore, segmentinfos is a private class.
     lucene::index::SegmentInfos infos;
     try {
