@@ -1,6 +1,6 @@
 /* This file is part of Strigi Desktop Search
  *
- * Copyright (C) 2006 Jos van den Oever <jos@vandenoever.info>
+ * Copyright (C) 2006,2009 Jos van den Oever <jos@vandenoever.info>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,10 +30,11 @@
 #include "streambase.h"
 #include "textutils.h"
 #include "strigi_thread.h"
+#include <time.h>
 #include <string>
 #include <cstdlib>
 #include <iconv.h>
-
+#include <cassert>
 #include <iostream>
 #include <map>
 
@@ -99,7 +100,7 @@ public:
     const time_t m_mtime;
     std::string m_name;
     const std::string m_path;
-    const std::string m_parentpath;
+    const std::string m_parentpath; // only use this value of m_parent == 0
     std::string m_encoding;
     std::string m_mimetype;
     IndexWriter& m_writer;
@@ -109,7 +110,8 @@ public:
     AnalysisResult* const m_this;
     AnalysisResult* const m_parent;
     const StreamEndAnalyzer* m_endanalyzer;
-    std::map<const Strigi::RegisteredField*, int> occurrences;
+    std::map<const Strigi::RegisteredField*, int> m_occurrences;
+    AnalysisResult* m_child;
 
     Private(const std::string& p, const char* name, time_t mt,
         AnalysisResult& t, AnalysisResult& parent);
@@ -127,12 +129,17 @@ AnalysisResult::Private::Private(const std::string& p, const char* name,
              m_indexer(parent.p->m_indexer),
              m_analyzerconfig(parent.p->m_analyzerconfig),
              m_this(&t), m_parent(&parent),
-             m_endanalyzer(0) {
+             m_endanalyzer(0), m_child(0) {
+    // make sure that the path starts with the path of the parent
+    assert(m_path.size() > m_parent->p->m_path.size()+1);
+    assert(m_path.compare(0, m_parent->p->m_path.size(), m_parent->p->m_path)
+        == 0);
 }
 AnalysisResult::AnalysisResult(const std::string& path, const char* name,
         time_t mt, AnalysisResult& parent)
         :p(new Private(path, name, mt, *this, parent)) {
     p->m_writer.startAnalysis(this);
+    srand(time(NULL));
 }
 AnalysisResult::Private::Private(const std::string& p, time_t mt,
         IndexWriter& w, StreamAnalyzer& indexer, const string& parentpath,
@@ -140,13 +147,30 @@ AnalysisResult::Private::Private(const std::string& p, time_t mt,
             :m_writerData(0), m_mtime(mt), m_path(p), m_parentpath(parentpath),
              m_writer(w), m_depth(0), m_indexer(indexer),
              m_analyzerconfig(indexer.configuration()), m_this(&t),
-             m_parent(0), m_endanalyzer(0) {
-    size_t pos = m_path.rfind('/');
+             m_parent(0), m_endanalyzer(0), m_child(0) {
+    size_t pos = m_path.rfind('/'); // TODO: perhaps us '\\' on Windows
     if (pos == std::string::npos) {
         m_name = m_path;
     } else {
+        if (pos == m_path.size()-1) {
+            // assert that there is no trailing '/' unless it is part of a
+            // protocol, which means the parent must be "" and the string must
+            // end in a colon followed by up to three slashes
+            assert(m_parentpath == "");
+            int i = m_path.size();
+            while (--i > 0 && m_path[i] == '/') {}
+            assert(i > 0 && m_path[i] == ':');
+        }
         m_name = m_path.substr(pos+1);
     }
+    // check that the path start with the path of the parent
+    // if the path of the parent is set (!= ""), m_path should be 2 characters
+    // longer: 1 for the separator and one for the file name.
+    // if m_path == "" and m_parentpath == "" then the unix root dir '/' is
+    // meant
+    assert((m_path.size() == 0 && m_parentpath.size() == 0)
+        || (m_path.size() > (m_parentpath.size()+(m_parentpath.size())?1:0)));
+    assert(m_path.compare(0, m_parentpath.size(), m_parentpath) == 0);
 }
 AnalysisResult::AnalysisResult(const std::string& path, time_t mt,
         IndexWriter& w, StreamAnalyzer& indexer, const string& parentpath)
@@ -154,6 +178,8 @@ AnalysisResult::AnalysisResult(const std::string& path, time_t mt,
     p->m_writer.startAnalysis(this);
 }
 AnalysisResult::~AnalysisResult() {
+    // delete child before writing and deleting the parent
+    delete p->m_child;
     p->write();
     delete p;
 }
@@ -162,7 +188,8 @@ AnalysisResult::Private::write() {
     const FieldRegister& fr = m_analyzerconfig.fieldRegister();
     m_writer.addValue(m_this, fr.pathField, m_path);
     // get the parent directory and store it without trailing slash
-    m_writer.addValue(m_this, fr.parentLocationField, m_parentpath);
+    m_writer.addValue(m_this, fr.parentLocationField,
+        (m_parent) ?m_parent->path() :m_parentpath);
 
     if (m_encoding.length()) {
         m_writer.addValue(m_this, fr.encodingField, m_encoding);
@@ -180,11 +207,16 @@ AnalysisResult::Private::write() {
     m_writer.addValue(m_this, fr.embeddepthField, (int32_t)m_depth);
     m_writer.addValue(m_this, fr.mtimeField, (uint32_t)m_mtime);
 
+    //FIXME a temporary workaround until we have a file(system) analyzer.
+    if(m_depth==0) m_writer.addValue(m_this, fr.typeField, "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#FileDataObject"); 
+    
     m_writer.finishAnalysis(m_this);
 }
 const std::string& AnalysisResult::fileName() const { return p->m_name; }
 const std::string& AnalysisResult::path() const { return p->m_path; }
-const string& AnalysisResult::parentPath() const { return p->m_parentpath; }
+const string& AnalysisResult::parentPath() const {
+    return (p->m_parent) ?p->m_parent->path() :p->m_parentpath;
+}
 time_t AnalysisResult::mTime() const { return p->m_mtime; }
 signed char AnalysisResult::depth() const { return p->m_depth; }
 int64_t AnalysisResult::id() const { return p->m_id; }
@@ -202,6 +234,10 @@ AnalysisResult::index(InputStream* file) {
 signed char
 AnalysisResult::indexChild(const std::string& name, time_t mt,
         InputStream* file) {
+    // clean up previous child
+    delete p->m_child;
+    p->m_child = 0;
+
     std::string path(p->m_path);
     path.append("/");
     path.append(name);
@@ -209,11 +245,15 @@ AnalysisResult::indexChild(const std::string& name, time_t mt,
     // check if we should index this file by applying the filename filters
     // make sure that the depth variable does not overflow
     if (depth() < 127 && p->m_analyzerconfig.indexFile(path.c_str(), n)) {
-        AnalysisResult i(path, n, mt, *this);
-        return p->m_indexer.analyze(i, file);
+        p->m_child = new AnalysisResult(path, n, mt, *this);
+        return p->m_indexer.analyze(*p->m_child, file);
     }
     return 0;
 }
+AnalysisResult*
+AnalysisResult::child() {
+    return p->m_child;
+}
 void
 AnalysisResult::addText(const char* text, int32_t length) {
     if (checkUtf8(text, length)) {
@@ -321,20 +361,36 @@ AnalysisResult::addValue(const RegisteredField* field, double value) {
 	return;
     p->m_writer.addValue(this, field, value);
 }
+void
+AnalysisResult::addTriplet(const std::string& subject, const std::string& predicate,
+        const std::string& object){
+    p->m_writer.addTriplet(subject, predicate, object);
+}
+std::string
+AnalysisResult::newAnonymousUri(){
+    std::string result;
+    result.resize(6);
+    result[0]=':';
+    for(int i=1; i<6; i++)
+      result[i]=(char)((rand() % 26) + 'a');
+    return result;
+}
 bool
 AnalysisResult::Private::checkCardinality(const RegisteredField* field) {
-    std::map<const Strigi::RegisteredField*, int>::const_iterator i = occurrences.find(field);
-    if (i != occurrences.end()) {
+    std::map<const Strigi::RegisteredField*, int>::const_iterator i
+        = m_occurrences.find(field);
+    if (i != m_occurrences.end()) {
 	if (i->second >= field->properties().maxCardinality()
                 && field->properties().maxCardinality() >= 0) {
 	    fprintf(stderr, "%s hit the maxCardinality limit (%d)\n",
-		field->properties().name().c_str(), field->properties().maxCardinality());
+		field->properties().name().c_str(),
+                    field->properties().maxCardinality());
 	    return false;
 	} else {
-	    occurrences[field]++;
+	    m_occurrences[field]++;
 	}
     } else {
-	occurrences[field] = 1;
+	m_occurrences[field] = 1;
     }
     return true;
 }
