@@ -27,19 +27,26 @@
 using namespace std;
 using namespace Strigi;
 
-const string BmpEndAnalyzerFactory::typeFieldName("http://freedesktop.org/standards/xesam/1.0/core#formatSubtype");
-const string BmpEndAnalyzerFactory::compressionFieldName("http://freedesktop.org/standards/xesam/1.0/core#compressionAlgorithm");
-const string BmpEndAnalyzerFactory::widthFieldName("http://freedesktop.org/standards/xesam/1.0/core#width");
-const string BmpEndAnalyzerFactory::heightFieldName("http://freedesktop.org/standards/xesam/1.0/core#height");
-const string BmpEndAnalyzerFactory::colorDepthFieldName("http://freedesktop.org/standards/xesam/1.0/core#pixelDataBitDepth");
-
 void
 BmpEndAnalyzerFactory::registerFields(FieldRegister& reg) {
-    typeField = reg.registerField(typeFieldName);
-    compressionField = reg.registerField(compressionFieldName);
-    widthField = reg.registerField(widthFieldName);
-    heightField = reg.registerField(heightFieldName);
-    colorDepthField = reg.registerField(colorDepthFieldName);
+    typeField = reg.registerField(
+        "http://freedesktop.org/standards/xesam/1.0/core#formatSubtype");
+    compressionField = reg.registerField(
+        "http://freedesktop.org/standards/xesam/1.0/core#compressionAlgorithm");
+    widthField = reg.registerField(
+        "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#width");
+    heightField = reg.registerField(
+        "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#height");
+    colorDepthField = reg.registerField(
+        "http://www.semanticdesktop.org/ontologies/nfo#colorDepth");
+    rdftypeField = reg.typeField;
+
+    addField(typeField);
+    addField(compressionField);
+    addField(widthField);
+    addField(heightField);
+    addField(colorDepthField);
+    addField(rdftypeField);
 }
 
 bool
@@ -55,7 +62,7 @@ BmpEndAnalyzer::checkHeader(const char* header, int32_t headersize) const {
     }
     return ok;
 }
-char
+signed char
 BmpEndAnalyzer::analyze(AnalysisResult& rs, InputStream* in) {
     // read BMP file type and ensure it is not damaged
     const char * bmptype_bm = "BM";
@@ -115,6 +122,8 @@ BmpEndAnalyzer::analyze(AnalysisResult& rs, InputStream* in) {
         rs.addValue(factory->compressionField, "Unknown");
     }
 
+    rs.addValue(factory->rdftypeField, "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#RasterImage");
+
     return 0;
 }
 
