@@ -40,7 +40,7 @@ Bz2EndAnalyzer::checkHeader(const char* header, int32_t headersize) const {
     bool v = BZ2InputStream::checkHeader(header, headersize);
     return v;
 }
-char
+signed char
 Bz2EndAnalyzer::analyze(AnalysisResult& idx, InputStream* in) {
     if(!in)
         return -1;
@@ -57,7 +57,7 @@ Bz2EndAnalyzer::analyze(AnalysisResult& idx, InputStream* in) {
         fprintf(stderr, "Error reading bz2: %s\n", stream.error());
         return -2;
     }
-    idx.addValue(factory->typeField, "http://freedesktop.org/standards/xesam/1.0/core#Archive");
+    idx.addValue(factory->typeField, "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#Archive");
     stream.reset(0);
     if (TarInputStream::checkHeader(start, nread)) {
         return TarEndAnalyzer::staticAnalyze(idx, &stream);
