@@ -39,7 +39,7 @@ GZipEndAnalyzer::checkHeader(const char* header, int32_t headersize) const {
     return headersize > 2 && (unsigned char)header[0] == 0x1f
         && (unsigned char)header[1] == 0x8b;
 }
-char
+signed char
 GZipEndAnalyzer::analyze(AnalysisResult& idx, InputStream* in) {
     if(!in)
         return -1;
@@ -53,7 +53,7 @@ GZipEndAnalyzer::analyze(AnalysisResult& idx, InputStream* in) {
         return -2;
     }
 
-    idx.addValue(factory->typeField, "http://freedesktop.org/standards/xesam/1.0/core#Archive");
+    idx.addValue(factory->typeField, "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#Archive");
 
     stream.reset(0);
     if (TarInputStream::checkHeader(start, nread)) {
