@@ -55,13 +55,13 @@ bool
 PdfEndAnalyzer::checkHeader(const char* header, int32_t headersize) const {
     return headersize > 7 && strncmp(header, "%PDF-1.", 7) == 0;
 }
-char
+signed char
 PdfEndAnalyzer::analyze(AnalysisResult& as, InputStream* in) {
     analysisresult = &as;
     n = 0;
     StreamStatus r = parser.parse(in);
     if (r != Eof) m_error.assign(parser.error());
     analysisresult->addValue(factory->typeField,
-        "http://freedesktop.org/standards/xesam/1.0/core#TextDocument");
+        "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#PaginatedTextDocument");
     return (r == Eof) ?0 :-1;
 }
