@@ -29,6 +29,16 @@
 #include <exiv2/exif.hpp>
 #include <exiv2/image.hpp>
 #include <exiv2/error.hpp>
+#include <math.h>
+#ifdef _MSC_VER
+// at least MSVC2008 needs this define for defines like M_LN2
+#define _USE_MATH_DEFINES
+#include <cmath>
+#endif
+
+#ifdef _MSC_VER
+#define HUGE_VALF HUGE_VAL
+#endif
 
 using namespace Strigi;
 using namespace std;
@@ -55,7 +65,7 @@ public:
         return "JpegEndAnalyzer";
     }
     bool checkHeader(const char* header, int32_t headersize) const;
-    char analyze(AnalysisResult& idx, ::InputStream* in);
+    signed char analyze(AnalysisResult& idx, ::InputStream* in);
 };
 
 /*
@@ -97,7 +107,6 @@ private:
     const RegisteredField* whiteBalanceField;
     const RegisteredField* meteringModeField;
     const RegisteredField* exposureField;
-    const RegisteredField* isoEquivField;
     const RegisteredField* jpegQualityField;
     const RegisteredField* userCommentField;
     const RegisteredField* jpegProcessField;
@@ -107,30 +116,34 @@ private:
 
 };
 
-const string JpegEndAnalyzerFactory::commentFieldName("http://freedesktop.org/standards/xesam/1.0/core#contentComment");
-const string JpegEndAnalyzerFactory::manufacturerFieldName("http://freedesktop.org/standards/xesam/1.0/core#cameraManufacturer");
-const string JpegEndAnalyzerFactory::modelFieldName("http://freedesktop.org/standards/xesam/1.0/core#cameraModel");
-const string JpegEndAnalyzerFactory::creationDateFieldName("http://freedesktop.org/standards/xesam/1.0/core#contentCreated");
-const string JpegEndAnalyzerFactory::widthFieldName("http://freedesktop.org/standards/xesam/1.0/core#width");
-const string JpegEndAnalyzerFactory::heightFieldName("http://freedesktop.org/standards/xesam/1.0/core#height");
-const string JpegEndAnalyzerFactory::orientationFieldName("http://freedesktop.org/standards/xesam/1.0/core#orientation");
-const string JpegEndAnalyzerFactory::colorModeFieldName("http://freedesktop.org/standards/xesam/1.0/core#colorSpace");
-const string JpegEndAnalyzerFactory::flashUsedFieldName("http://freedesktop.org/standards/xesam/1.0/core#flashUsed");
-const string JpegEndAnalyzerFactory::focalLengthFieldName("http://freedesktop.org/standards/xesam/1.0/core#focalLength");
-const string JpegEndAnalyzerFactory::_35mmEquivalentFieldName("http://freedesktop.org/standards/xesam/1.0/core#35mmEquivalent");
-const string JpegEndAnalyzerFactory::ccdWidthFieldName("http://freedesktop.org/standards/xesam/1.0/core#ccdWidth");
-const string JpegEndAnalyzerFactory::exposureTimeFieldName("http://freedesktop.org/standards/xesam/1.0/core#exposureTime");
-const string JpegEndAnalyzerFactory::apertureFieldName("http://freedesktop.org/standards/xesam/1.0/core#aperture");
-const string JpegEndAnalyzerFactory::focusDistFieldName("http://freedesktop.org/standards/xesam/1.0/core#focusDistance");
-const string JpegEndAnalyzerFactory::exposureBiasFieldName("http://freedesktop.org/standards/xesam/1.0/core#exposureBias");
-const string JpegEndAnalyzerFactory::whiteBalanceFieldName("http://freedesktop.org/standards/xesam/1.0/core#whiteBalance");
-const string JpegEndAnalyzerFactory::meteringModeFieldName("http://freedesktop.org/standards/xesam/1.0/core#meteringMode");
-const string JpegEndAnalyzerFactory::exposureFieldName("http://freedesktop.org/standards/xesam/1.0/core#exposureProgram");
-const string JpegEndAnalyzerFactory::isoEquivFieldName("http://freedesktop.org/standards/xesam/1.0/core#isoEquivalent");
-const string JpegEndAnalyzerFactory::jpegQualityFieldName("http://freedesktop.org/standards/xesam/1.0/core#targetQuality");
-const string JpegEndAnalyzerFactory::userCommentFieldName("http://freedesktop.org/standards/xesam/1.0/core#userComment");
-const string JpegEndAnalyzerFactory::jpegProcessFieldName("http://freedesktop.org/standards/xesam/1.0/core#compressionAlgorithm");
-const string JpegEndAnalyzerFactory::thumbnailFieldName("content.thumbnail");
+#define NEXIF "http://www.semanticdesktop.org/ontologies/2007/05/10/nexif#"
+
+const string commentFieldName("http://www.semanticdesktop.org/ontologies/2007/01/19/nie#comment");
+const string manufacturerFieldName(NEXIF "make");
+const string modelFieldName(NEXIF "model");
+const string creationDateFieldName("http://www.semanticdesktop.org/ontologies/2007/01/19/nie#contentCreated");
+const string widthFieldName("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#width");
+const string heightFieldName("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#height");
+const string orientationFieldName(NEXIF "orientation");
+const string colorModeFieldName("http://freedesktop.org/standards/xesam/1.0/core#colorSpace");
+const string flashUsedFieldName(NEXIF "flash");
+const string focalLengthFieldName(NEXIF "focalLength");
+const string _35mmEquivalentFieldName(NEXIF "focalLengthIn35mmFilm");
+const string ccdWidthFieldName("http://freedesktop.org/standards/xesam/1.0/core#ccdWidth");
+const string exposureTimeFieldName(NEXIF "exposureTime");
+const string apertureFieldName(NEXIF "apertureValue");
+const string focusDistFieldName("http://freedesktop.org/standards/xesam/1.0/core#focusDistance");
+const string exposureBiasFieldName(NEXIF "exposureBiasValue");
+const string whiteBalanceFieldName(NEXIF "whiteBalance");
+const string meteringModeFieldName(NEXIF "meteringMode");
+const string exposureFieldName(NEXIF "exposureProgram");
+const string jpegQualityFieldName("http://freedesktop.org/standards/xesam/1.0/core#targetQuality");
+const string userCommentFieldName("http://www.semanticdesktop.org/ontologies/2007/01/19/nie#comment");
+const string jpegProcessFieldName("http://freedesktop.org/standards/xesam/1.0/core#compressionAlgorithm");
+const string thumbnailFieldName("http://strigi.sf.net/ontologies/homeless#contentThumbnail");
+const string ISOSpeedRatingsFieldName(NEXIF "isoSpeedRatings");
+
+#undef NEXIF
 
 /*
  Register the field names so that the StreamIndexer knows which analyzer
@@ -138,68 +151,44 @@ const string JpegEndAnalyzerFactory::thumbnailFieldName("content.thumbnail");
 */
 void
 JpegEndAnalyzerFactory::registerFields(FieldRegister& r) {
-    commentField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#contentComment");
+    commentField = r.registerField(commentFieldName);
     addField(commentField);
 
-    exifFields["Exif.Image.DateTime"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#contentCreated");
-    exifFields["Exif.Image.Make"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#cameraManufacturer");
-    exifFields["Exif.Image.Model"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#cameraModel");
-    exifFields["Exif.Photo.PixelXDimension"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#width");
-    exifFields["Exif.Photo.PixelYDimension"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#height");
-    exifFields["Exif.Image.Orientation"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#orientation");
-    exifFields["Exif.Photo.Flash"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#flashUsed");
-    exifFields["Exif.Photo.FocalLength"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#focalLength");
-    exifFields["Exif.Photo.FocalLengthIn35mmFilm"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#35mmEquivalent");
-    exifFields["Exif.Photo.ExposureTime"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#exposureTime");
-    exifFields["Exif.Photo.ApertureValue"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#aperture");
-    exifFields["Exif.Photo.ExposureBiasValue"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#exposureBias");
-    exifFields["Exif.Photo.WhiteBalance"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#whiteBalance");
-    exifFields["Exif.Photo.MeteringMode"] = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#meteringMode");
-    map<string, const RegisteredField*>::const_iterator i = exifFields.begin();
+    exifFields["Exif.Image.DateTime"] = r.registerField(creationDateFieldName);
+    exifFields["Exif.Image.Make"] = r.registerField(manufacturerFieldName);
+    exifFields["Exif.Image.Model"] = r.registerField(modelFieldName);
+    exifFields["Exif.Photo.PixelXDimension"] = r.registerField(widthFieldName);
+    exifFields["Exif.Photo.PixelYDimension"] = r.registerField(heightFieldName);
+    exifFields["Exif.Image.Orientation"] = r.registerField(orientationFieldName);
+    exifFields["Exif.Photo.Flash"] = r.registerField(flashUsedFieldName);
+    exifFields["Exif.Photo.FocalLength"] = r.registerField(focalLengthFieldName);
+    exifFields["Exif.Photo.FocalLengthIn35mmFilm"] = r.registerField(_35mmEquivalentFieldName);
+    exifFields["Exif.Photo.ExposureTime"] = r.registerField(exposureTimeFieldName);
+    exifFields["Exif.Photo.ApertureValue"] = r.registerField(apertureFieldName);
+    exifFields["Exif.Photo.ExposureBiasValue"] = r.registerField(exposureBiasFieldName);
+    exifFields["Exif.Photo.WhiteBalance"] = r.registerField(whiteBalanceFieldName);
+    exifFields["Exif.Photo.MeteringMode"] = r.registerField(meteringModeFieldName);
+    exifFields["Exif.Photo.ISOSpeedRatings"] = r.registerField(ISOSpeedRatingsFieldName);
+
+map<string, const RegisteredField*>::const_iterator i = exifFields.begin();
     for (; i != exifFields.end(); ++i) {
         addField(i->second);
     }
 
-    colorModeField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#colorSpace");
-    ccdWidthField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#ccdWidth");
-    focusDistField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#focusDistance");
-    exposureField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#exposureProgram");
-    isoEquivField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#isoEquivalent");
-    jpegQualityField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#targetQuality");
-    userCommentField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#userComment");
-    jpegProcessField = r.registerField(
-        "http://freedesktop.org/standards/xesam/1.0/core#compressionAlgorithm");
-    thumbnailField = r.registerField(
-        "content.thumbnail");
+    colorModeField = r.registerField(colorModeFieldName);
+    ccdWidthField = r.registerField(ccdWidthFieldName);
+    focusDistField = r.registerField(focusDistFieldName);
+    exposureField = r.registerField(exposureFieldName);
+    jpegQualityField = r.registerField(jpegQualityFieldName);
+    userCommentField = r.registerField(userCommentFieldName);
+    jpegProcessField = r.registerField(jpegProcessFieldName);
+    thumbnailField = r.registerField(thumbnailFieldName);
     typeField = r.typeField;
 
     addField(colorModeField);
     addField(ccdWidthField);
     addField(focusDistField);
     addField(exposureField);
-    addField(isoEquivField);
     addField(jpegQualityField);
     addField(userCommentField);
     addField(jpegProcessField);
@@ -213,7 +202,32 @@ JpegEndAnalyzer::checkHeader(const char* header, int32_t headersize) const {
         = {0xFF, 0xD8, 0xFF};
     return headersize >= 3 &&  memcmp(header, jpgmagic, 3) == 0;
 }
-char
+namespace {
+
+bool
+fnumberToApertureValue(string& fnumber) {
+    // Convert the F number to the aperture value
+    // return true if the value in fnumber is changed to ApertureValue
+    // F number = sqrt(pow(2, apex))
+    char* end;
+    double c = strtod(fnumber.c_str(), &end);
+    if (c <= 0 || c == HUGE_VALF || *end == '\0') return false;
+    double d = strtod(end + 1, &end);
+    if (d <= 0 || d == HUGE_VALF) return false;
+    c /= d;
+
+    // do not use log2() or round() here to be cross-platform
+    double apex = log(c*c) / M_LN2 * 65536;
+    apex  = (apex < 0 ? ceil(apex - 0.5) : floor(apex + 0.5));
+    std::ostringstream aperture;
+    aperture << apex << "/65536";
+    fnumber.assign(aperture.str());
+    return true;
+}
+
+}
+
+signed char
 JpegEndAnalyzer::analyze(AnalysisResult& ar, ::InputStream* in) {
     // parse the jpeg file now
     Exiv2::Image::AutoPtr img;
@@ -243,7 +257,7 @@ JpegEndAnalyzer::analyze(AnalysisResult& ar, ::InputStream* in) {
             m_error.assign("no valid jpeg");
             return -1;
         }
-    
+
         try {
             const Exiv2::byte* d = (const Exiv2::byte*)data;
             img = Exiv2::ImageFactory::open(d, nread);
@@ -256,6 +270,11 @@ JpegEndAnalyzer::analyze(AnalysisResult& ar, ::InputStream* in) {
         } catch (Exiv2::AnyError& e) {
             m_error.assign(e.what());
             return -1;
+        } catch(std::exception& e) {
+            // trueg: here Exiv2::Error is not caught above, until the reason is found
+            // I added this generic exception handling which should do no harm whatsoever
+            m_error.assign(e.what());
+            return -1;
         }
     }
 
@@ -265,13 +284,22 @@ JpegEndAnalyzer::analyze(AnalysisResult& ar, ::InputStream* in) {
 
     const Exiv2::ExifData& exif = img->exifData();
     // if there's exif data, this is a photo, otherwise just an image
-    if( exif.size() ) {
-        ar.addValue(factory->typeField, "http://freedesktop.org/standards/xesam/1.0/core#Photo");
+    if( ! exif.empty() ) {
+        ar.addValue(factory->typeField, "http://www.semanticdesktop.org/ontologies/2007/05/10/nexif#Photo");
     } else {
-        ar.addValue(factory->typeField, "http://freedesktop.org/standards/xesam/1.0/core#Image");
+        ar.addValue(factory->typeField, "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#RasterImage");
     }
 
-    for (Exiv2::ExifData::const_iterator i = exif.begin(); i != exif.end();i++){
+    for (Exiv2::ExifData::const_iterator i = exif.begin(); i != exif.end();++i){
+        if (i->key() == "Exif.Photo.FNumber") {
+            string aperture(i->toString());
+            if (fnumberToApertureValue(aperture)) {
+                ar.addValue(
+                   factory->exifFields.find("Exif.Photo.ApertureValue")->second,
+                   aperture);
+            }
+            continue;
+        }
         map<string,const RegisteredField*>::const_iterator f
             = factory->exifFields.find(i->key());
         if (f != factory->exifFields.end() && f->second) {
@@ -280,14 +308,23 @@ JpegEndAnalyzer::analyze(AnalysisResult& ar, ::InputStream* in) {
 //            cerr << i->key() << "\t" << i->value() << endl;
         }
     }
-
+#if (EXIV2_TEST_VERSION(0,17,91))
+    Exiv2::ExifThumbC thumb(exif);
+    Exiv2::DataBuf thumbnail = thumb.copy();
+#else
     Exiv2::DataBuf thumbnail = exif.copyThumbnail();
+#endif
     data = (const char*)thumbnail.pData_;
     if (data) {
         StringInputStream thumbstream(data, thumbnail.size_, false);
         string thumbname("thumbnail");
+#if (EXIV2_TEST_VERSION(0,17,91))
+        ar.indexChild(thumbname + thumb.extension(), ar.mTime(),
+            &thumbstream);
+#else
         ar.indexChild(thumbname + exif.thumbnailExtension(), ar.mTime(),
             &thumbstream);
+#endif
     }
 
 /*
