@@ -22,6 +22,11 @@
 #include "fieldproperties_private.h"
 #include "fieldtypes.h"
 #include <vector>
+#include <map>
+#include <iostream>
+#include <set>
+#include <cstdlib>
+#include <cstring>
 #include <sys/types.h>
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
@@ -34,12 +39,6 @@
 #include <libxml/SAX2.h>
 #include <sys/stat.h>
 #include <config.h>
-#include <cstdlib>
-#include <cstring>
-#include <map>
-#include <iostream>
-#include <list>
-#include <set>
 
 using namespace Strigi;
 using namespace std;
@@ -57,9 +56,9 @@ public:
     static vector<string> getXdgDirs();
     void addEssentialProperties();
     void loadProperties(const string& dir);
-    void parseProperties(char*data);
+    void parseProperties(FILE* f);
     void storeProperties(FieldProperties::Private& props);
-    void warnIfLocale(const char* name, string& locale);
+    void warnIfLocale(const char* name, const string& locale);
 
 // SAX Callbacks and stuff
     map<string, FieldProperties::Private> pProperties;
@@ -71,12 +70,13 @@ public:
     string currentElementChars;
     string currentElementLang;
     string currentElementResource;
+    bool nestedResource;
     FieldProperties::Private currentField;
     ClassProperties::Private currentClass;
-    map<string,xmlEntity*> xmlEntities;
-    list< pair<string, string> > entities;
+    map<string, xmlEntity> xmlEntities;
 
-    void setDefinitionAttribute(const char * name, const char * value);
+    void setDefinitionAttribute(const char* name, size_t namelen,
+        const char * value, size_t valuelen);
 
     static void charactersSAXFunc(void* ctx, const xmlChar * ch, int len);
     static void errorSAXFunc(void* ctx, const char * msg, ...);
@@ -92,8 +92,6 @@ public:
 
     static bool isBoolValid(const char *uri, const char* name,
         const char* value, bool& result);
-    void replaceEntities(string& value);
-
 };
 
 const FieldProperties&
@@ -263,7 +261,9 @@ FieldPropertiesDb::Private::Private() {
             if(propertiesByAlias.find(alias) == propertiesByAlias.end()) {
                 propertiesByAlias[alias] = property;
             } else {
-                cerr << "Error: alias " << alias << " requested by several properties" << endl;
+                cerr << "Error: alias " << alias << " requested by several properties: " << propertiesByAlias.find(alias)->second.uri()
+		  << ", " << prop->second.uri
+		  << endl;
             }
         }
 
@@ -296,16 +296,18 @@ FieldPropertiesDb::Private::addEssentialProperties() {
 
     props.typeuri = FieldRegister::stringType;
     props.uri = FieldRegister::pathFieldName;
+    props.tokenized = false; // should not be tokenized: needed for retrieval 
     properties[FieldRegister::pathFieldName] = props;
 
     props.uri = FieldRegister::filenameFieldName;
+    props.tokenized = true;
     properties[FieldRegister::filenameFieldName] = props;
 
     props.uri = FieldRegister::mimetypeFieldName;
     properties[FieldRegister::mimetypeFieldName] = props;
 
     props.uri = FieldRegister::parentLocationFieldName;
-    props.tokenized = false;
+    props.tokenized = false; // should not be tokenized: needed for retrieval
     properties[FieldRegister::parentLocationFieldName] = props;
 }
 void
@@ -324,30 +326,33 @@ FieldPropertiesDb::Private::loadProperties(const string& dir) {
     }
     struct dirent* de = readdir(d);
     struct stat s;
-    char* data = 0;
     while (de) {
         string path(pdir+de->d_name);
-        if (path.length() >= 5 && path.substr(path.length()-5) == ".rdfs" &&
+        if (path.length() >= 5 && path.compare(path.length() - 5, 5, ".rdfs", 5) == 0 &&
                 !stat(path.c_str(), &s) && S_ISREG(s.st_mode)) {
             FILE* f = fopen(path.c_str(), "r");
             if (f) {
-                // read the entire file at once
-                data = (char*)realloc(data, s.st_size+1);
-                if (fread(data, 1, s.st_size, f) == (size_t)s.st_size) {
-                    //fprintf(stderr, "parsing %s\n", path.c_str());
-                    data[s.st_size] = '\0';
-                    parseProperties(data);
-                }
+                parseProperties(f);
                 fclose(f);
             }
         }
         de = readdir(d);
     }
     closedir(d);
-    free(data);
+}
+namespace {
+int
+strigi_xmlFileRead(void* context, char* buffer, int len) {
+    FILE* f = (FILE*)context;
+    return fread(buffer, 1, len, f);
+}
+int
+strigi_xmlFileClose(void*) {
+    return 0;
+}
 }
 void
-FieldPropertiesDb::Private::parseProperties(char* data) {
+FieldPropertiesDb::Private::parseProperties(FILE* f) {
     FieldProperties::Private props;
 
     xmlParserCtxtPtr ctxt;
@@ -368,24 +373,29 @@ FieldPropertiesDb::Private::parseProperties(char* data) {
     currentField.clear();
     currentClass.clear();
     currentDefinition = defNone;
-
-    ctxt = xmlCreatePushParserCtxt(&handler, this, data, strlen(data), "xxx");
-    if (ctxt == 0 || xmlParseChunk(ctxt, 0, 0, 1)) {
+    nestedResource = false;
+    
+    ctxt = xmlCreateIOParserCtxt(&handler, this, strigi_xmlFileRead, strigi_xmlFileClose, f, XML_CHAR_ENCODING_NONE);
+    if (ctxt == 0) {
         saxError = true;
+    } else {
+        xmlCtxtUseOptions(ctxt, XML_PARSE_NOENT);
+        if (xmlParseDocument(ctxt)) {
+            saxError = true;
+        }
     }
 
     if(saxError) {
         cerr << "saxError in FieldPropertiesDB::parseProperties." << endl;
     }
 
+    xmlFreeDoc(ctxt->myDoc);
     xmlFreeParserCtxt(ctxt);
-    entities.clear();
 
-    for (map<std::string, xmlEntityPtr>::iterator j=xmlEntities.begin();
+    for (map<std::string, xmlEntity>::iterator j=xmlEntities.begin();
             j!=xmlEntities.end(); ++j) {
-        delete [] j->second->name;
-        delete [] j->second->content;
-        delete j->second;
+        delete [] j->second.name;
+        delete [] j->second.content;
     }
     xmlEntities.clear();
 }
@@ -394,32 +404,20 @@ FieldPropertiesDb::Private::xmlSAX2EntityDecl(void * ctx, const xmlChar * name,
         int type, const xmlChar* publicId, const xmlChar* systemId,
         xmlChar* content) {
     Private* p = (Private*)ctx;
-    xmlEntityPtr newEntity;
-    char * str;
-    int len;
-    map<std::string, xmlEntityPtr>::const_iterator j
-        = p->xmlEntities.find((const char *)name);
+    string stdname((const char*)name);
+    map<std::string, xmlEntity>::const_iterator j
+        = p->xmlEntities.find(stdname);
     if (j == p->xmlEntities.end()) {
-        newEntity = new xmlEntity;
-        memset(newEntity, 0 , sizeof(xmlEntity));
-        newEntity->type = XML_ENTITY_DECL;
-
-        len=strlen((const char*)name);
-        str = new char[len+1];
-        strcpy(str, (const char*)name);
-        newEntity->name = (const xmlChar *)str;
-        len = strlen((const char *)content);
-        newEntity->length = len;
-        str = new char[len+1];
-        strcpy(str, (const char *)content);
-        newEntity->orig = (xmlChar *)str;
-        newEntity->content = (xmlChar *)str;
-        newEntity->etype = XML_INTERNAL_GENERAL_ENTITY;
-        newEntity->URI = (xmlChar *)str;
-
-        p->xmlEntities[(const char *)name] = newEntity;
-        p->entities.push_back(
-            pair<string,string>((const char *)name, (const char *)content));
+        xmlEntity& newEntity = p->xmlEntities[stdname];
+        newEntity.type = XML_ENTITY_DECL;
+        newEntity.name = (xmlChar*)new char[stdname.size()+1];
+        strcpy((char*)newEntity.name, stdname.c_str());
+        newEntity.length = strlen((const char*)content);
+        newEntity.orig = (xmlChar*)new char[newEntity.length+1];
+        strcpy((char*)newEntity.orig, (const char*)content);
+        newEntity.content = newEntity.orig;
+        newEntity.etype = XML_INTERNAL_GENERAL_ENTITY;
+        newEntity.URI = newEntity.orig;
     } else {
         cerr << "Error: entity " << name << " redeclared." << endl;
     }
@@ -428,12 +426,12 @@ FieldPropertiesDb::Private::xmlSAX2EntityDecl(void * ctx, const xmlChar * name,
 xmlEntityPtr
 FieldPropertiesDb::Private::getEntitySAXFunc(void * ctx, const xmlChar * name) {
     Private* p = (Private*)ctx;
-    map<std::string, xmlEntityPtr>::const_iterator j
+    map<std::string, xmlEntity>::iterator j
         = p->xmlEntities.find((const char *)name);
     if (j == p->xmlEntities.end()) {
         return NULL;
     } else {
-        return j->second;
+        return &j->second;
     }
 }
 
@@ -472,34 +470,24 @@ FieldPropertiesDb::Private::isBoolValid(const char *uri, const char* name,
     }
     return true;
 }
-
-// workaround for entities handling problem with libxml
-void
-FieldPropertiesDb::Private::replaceEntities(string& value) {
-    for (list< pair<string, string> >::const_iterator j = entities.begin();
-            j != entities.end(); ++j) {
-        string entity = '&' + j->first + ';';
-        size_t pos = string::npos;
-        while((pos = value.find(entity)) != string::npos) {
-        value.erase(pos, entity.size());
-            value.insert(pos, j->second);
-        }
-    }
+namespace {
+// due to inlining, the strlen(B) is evaluated at runtime when B is a literal
+// string
+inline bool compare(const char* a, size_t alen, const char* b) {
+    return alen == strlen(b) && strncmp(a, b, strlen(b)) == 0;
+}
 }
-
 void
-FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
-         const char * value) {
+FieldPropertiesDb::Private::setDefinitionAttribute(const char* name,
+         size_t namelen, const char* value, size_t valuelen) {
     bool boolValue;
     //Trim leading and trailing whitespace
-    string val = value;
-    val.erase(0, val.find_first_not_of(" \t\n"));
+    size_t trimmedvallen = valuelen - strspn(value, " \t\n");
+    string val(value + valuelen - trimmedvallen, trimmedvallen);
     val.erase(val.find_last_not_of(" \t\n") + 1);
 
-    replaceEntities(val);
-
     if (currentDefinition == defProperty) {
-        if (strcmp(name, "about") == 0) {
+        if (compare(name, namelen, "about")) {
             warnIfLocale(val.c_str(), currentElementLang);
             if (currentField.uri.size()) {
                 cerr << "Uri is already defined for " << currentField.uri << "."
@@ -507,7 +495,7 @@ FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
             } else {
                 currentField.uri.assign(val);
             }
-        } else if (strcmp(name, "alias") == 0) {
+        } else if (compare(name, namelen, "alias")) {
             warnIfLocale(val.c_str(), currentElementLang);
             if (currentField.alias.size()) {
                 cerr << "alias is already defined for " << currentField.uri << "."
@@ -515,16 +503,15 @@ FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
             } else {
                 currentField.alias.assign(val);
             }
-        } else if(strcmp(name, "range") == 0) {
+        } else if (compare(name, namelen, "range")) {
             warnIfLocale(currentField.uri.c_str(), currentElementLang);
             if (currentField.typeuri.size()) {
                cerr << "range is already defined for " << currentField.uri
                    << "." << endl;
             } else {
-                replaceEntities(currentElementResource);
                 currentField.typeuri.assign(currentElementResource);
             }
-        } else if(strcmp(name, "label") == 0) {
+        } else if (compare(name, namelen, "label")) {
             if (currentElementLang.size()) {
                 FieldProperties::Localized l(
                     currentField.localized[currentElementLang]);
@@ -542,7 +529,7 @@ FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
             } else {
                 currentField.name.assign(val);
             }
-        } else if (strcmp(name, "comment") == 0) {
+        } else if (compare(name, namelen, "comment")) {
             if (currentElementLang.size()) {
                 FieldProperties::Localized l(
                     currentField.localized[currentElementLang]);
@@ -560,44 +547,42 @@ FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
             } else {
                 currentField.description.assign(val);
             }
-        } else if (strcmp(name, "subPropertyOf") == 0) {
-            replaceEntities(currentElementResource);
+        } else if (compare(name, namelen, "subPropertyOf")) {
             currentField.parentUris.push_back(currentElementResource);
-        } else if (strcmp(name, "domain") == 0) {
-            replaceEntities(currentElementResource);
+        } else if (compare(name, namelen, "domain")) {
             currentField.applicableClasses.push_back(currentElementResource);
-        } else if (strcmp(name, "binary") == 0) {
+        } else if (compare(name, namelen, "binary")) {
             if (isBoolValid(currentField.uri.c_str(), "binary", value,
                     boolValue)) {
                 currentField.binary = boolValue;
             }
-        } else if (strcmp(name, "compressed") == 0) {
+        } else if (compare(name, namelen, "compressed")) {
             if (isBoolValid(currentField.uri.c_str(), "compressed", value,
                     boolValue)) {
                 currentField.compressed = boolValue;
             }
-        } else if (strcmp(name, "indexed") == 0) {
+        } else if (compare(name, namelen, "indexed")) {
             if (isBoolValid(currentField.uri.c_str(), "indexed", value,
                     boolValue)) {
                 currentField.indexed = boolValue;
             }
-        } else if (strcmp(name, "stored") == 0) {
+        } else if (compare(name, namelen, "stored")) {
             if (isBoolValid(currentField.uri.c_str(), "stored", value,
                     boolValue)) {
                 currentField.stored = boolValue;
             }
-        } else if (strcmp(name, "tokenized") == 0) {
+        } else if (compare(name, namelen, "tokenized")) {
             if (isBoolValid(currentField.uri.c_str(), "tokenized", value,
                     boolValue)) {
                 currentField.tokenized = boolValue;
             }
-        } else if (strcmp(name, "minCardinality") == 0) {
+        } else if (compare(name, namelen, "minCardinality")) {
             currentField.min_cardinality = atoi(value);
-        } else if (strcmp(name, "maxCardinality") == 0) {
+        } else if (compare(name, namelen, "maxCardinality")) {
             currentField.max_cardinality = atoi(value);
         }
     } else if (currentDefinition == defClass) {
-        if (strcmp(name, "about") == 0) {
+        if (compare(name, namelen, "about")) {
             warnIfLocale(val.c_str(), currentElementLang);
             if (currentClass.uri.size()) {
                 cerr << "Uri is already defined for " << currentClass.uri
@@ -606,7 +591,7 @@ FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
         } else {
             currentClass.uri.assign(val);
         }
-    } else if (strcmp(name, "label") == 0) {
+    } else if (compare(name, namelen, "label")) {
         if (currentElementLang.size()) {
             ClassProperties::Localized l(
                 currentClass.localized[currentElementLang]);
@@ -624,7 +609,7 @@ FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
         } else {
             currentClass.name.assign(val);
         }
-    } else if (strcmp(name, "comment") == 0) {
+    } else if (compare(name, namelen, "comment")) {
         if (currentElementLang.size()) {
             ClassProperties::Localized l(
                     currentClass.localized[currentElementLang]);
@@ -642,8 +627,7 @@ FieldPropertiesDb::Private::setDefinitionAttribute(const char * name,
         } else {
             currentClass.description.assign(val);
         }
-    } else if(strcmp(name, "subClassOf") == 0) {
-        replaceEntities(currentElementResource);
+    } else if(compare(name, namelen, "subClassOf")) {
         currentClass.parentUris.push_back(currentElementResource);
     }
 }
@@ -663,17 +647,27 @@ FieldPropertiesDb::Private::startElementNsSAX2Func(void * ctx,
         }
         if (p->currentDefinition != defNone) {
             for (int i=0; i<nb_attributes; i++) {
-                p->setDefinitionAttribute((const char*)attributes[i*5],
-                    (const char*)attributes[3+i*5]);
+                const xmlChar ** a = attributes + i*5;
+                p->setDefinitionAttribute((const char*)a[0],
+                    strlen((const char*)a[0]), (const char*)a[3], a[4]-a[3]);
             }
         }
     } else {
-        p->currentSubElement = (const char *)localname;
+	if ( (strcmp((const char *)localname, "Property") == 0) ||
+	      (strcmp((const char *)localname, "Class") == 0) ) {
+	    p->nestedResource = true;
+	} else {
+	  p->currentSubElement = (const char *)localname;
+	}
         for (int i=0; i<nb_attributes; i++) {
-            if (strcmp((const char*)attributes[i*5], "resource") == 0) {
-                p->currentElementResource = (const char*)attributes[3+i*5];
+            const xmlChar ** a = attributes + i*5;
+            size_t len = a[0]-a[1];
+            if (len == 8 && strncmp((const char*)attributes[i*5], "resource", 8) == 0) {
+                p->currentElementResource.assign((const char*)a[3], a[4]-a[3]);
+            } else if (strcmp((const char*)attributes[i*5], "about") == 0) {
+                p->currentElementResource.assign((const char*)a[3], a[4]-a[3]);
             } else if (strcmp((const char*)attributes[i*5], "lang") == 0) {
-                p->currentElementLang = (const char*)attributes[3+i*5];
+                p->currentElementLang.assign((const char*)a[3], a[4]-a[3]);
             }
         }
     }
@@ -686,27 +680,37 @@ FieldPropertiesDb::Private::endElementNsSAX2Func(void *ctx,
 
     if (p->currentDefinition!=defNone) {
         if (strcmp((const char *)localname, "Property") == 0) {
-            if (p->currentField.uri.size()) {
-                if(!p->currentField.alias.size()) {
-                    size_t pos;
-                    if ((pos = p->currentField.uri.rfind('#')) != string::npos){
-                        p->currentField.alias = p->currentField.uri.substr(pos+1);
-                    }
-                }
-                p->pProperties[p->currentField.uri] = p->currentField;
-                p->currentField.clear();
-            }
-            p->currentDefinition = defNone;
+	    if (p->nestedResource) {
+		p->nestedResource = false;
+	    } else {
+		if (p->currentField.uri.size()) {
+		    if(!p->currentField.alias.size()) {
+			size_t pos;
+			if ((pos = p->currentField.uri.rfind('#')) != string::npos){
+			    p->currentField.alias = p->currentField.uri.substr(pos+1);
+			}
+		    }
+		    p->pProperties[p->currentField.uri] = p->currentField;
+		    p->currentField.clear();
+		}
+		p->currentDefinition = defNone;
+	    }
         } else if (strcmp((const char *)localname, "Class") == 0) {
-            if (p->currentClass.uri.size()) {
-                p->pClasses[p->currentClass.uri] = p->currentClass;
-                p->currentClass.clear();
-            }
-            p->currentDefinition = defNone;
+	    if (p->nestedResource) {
+		p->nestedResource = false;
+	    } else {
+		if (p->currentClass.uri.size()) {
+		    p->pClasses[p->currentClass.uri] = p->currentClass;
+		    p->currentClass.clear();
+		}
+		p->currentDefinition = defNone;
+	    }
         } else {
             if (p->currentSubElement == (const char*)localname) {
                 p->setDefinitionAttribute(p->currentSubElement.c_str(),
-                p->currentElementChars.c_str());
+                    p->currentSubElement.size(),
+                    p->currentElementChars.c_str(),
+                    p->currentElementChars.size());
                 p->currentSubElement = "";
                 p->currentElementChars = "";
                 p->currentElementResource = "";
@@ -728,7 +732,7 @@ FieldPropertiesDb::Private::storeProperties(FieldProperties::Private& p) {
 }
 
 void
-FieldPropertiesDb::Private::warnIfLocale(const char* name, string& locale) {
+FieldPropertiesDb::Private::warnIfLocale(const char* name, const string& locale) {
     if (locale.size()) {
         cerr << "Warning: you cannot define a locale for the resource URI "
             << name << "." << endl;
