@@ -28,19 +28,27 @@ using namespace std;
 using namespace Strigi;
 
 // AnalyzerFactory
-const string TgaThroughAnalyzerFactory::compressionFieldName("compressed.compression_algorithm");
-const string TgaThroughAnalyzerFactory::colorDepthFieldName("http://freedesktop.org/standards/xesam/1.0/core#pixelDataBitDepth");
-const string TgaThroughAnalyzerFactory::colorModeFieldName("http://freedesktop.org/standards/xesam/1.0/core#colorSpace");
-const string TgaThroughAnalyzerFactory::widthFieldName("http://freedesktop.org/standards/xesam/1.0/core#width");
-const string TgaThroughAnalyzerFactory::heightFieldName("http://freedesktop.org/standards/xesam/1.0/core#height");
 
 void
 TgaThroughAnalyzerFactory::registerFields(FieldRegister& reg) {
-    compressionField = reg.registerField(compressionFieldName);
-    colorDepthField = reg.registerField(colorDepthFieldName);
-    colorModeField = reg.registerField(colorModeFieldName);
-    widthField = reg.registerField(widthFieldName);
-    heightField = reg.registerField(heightFieldName);
+    compressionField = reg.registerField(
+        "http://freedesktop.org/standards/xesam/1.0/core#compressionAlgorithm");
+    colorDepthField = reg.registerField(
+        "http://www.semanticdesktop.org/ontologies/nfo#colorDepth");
+    colorModeField = reg.registerField(
+        "http://freedesktop.org/standards/xesam/1.0/core#colorSpace");
+    widthField = reg.registerField(
+        "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#width");
+    heightField = reg.registerField(
+        "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#height");
+    typeField = reg.typeField;
+
+    addField(compressionField);
+    addField(colorDepthField);
+    addField(colorModeField);
+    addField(widthField);
+    addField(heightField);
+    addField(typeField);
 }
 
 // Analyzer
@@ -110,6 +118,7 @@ TgaThroughAnalyzer::connectInputStream(InputStream* in) {
 
     uint8_t colorDepth = *(buf+16);
     analysisResult->addValue(factory->colorDepthField, colorDepth);
+    analysisResult->addValue(factory->typeField, "http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#RasterImage");
 
     return in;
 }
