@@ -19,8 +19,11 @@
  */
 #include "indexmanagertester.h"
 #include "indexmanager.h"
+#include "indexwriter.h"
 #include "indexpluginloader.h"
 
+#include <stdlib.h>
+
 using namespace std;
 using namespace Strigi;
 using namespace strigiunittest;
@@ -47,12 +50,26 @@ IndexManagerTest::tearDown() {
 
 void
 IndexManagerTest::testIndexReader() {
+    CPPUNIT_ASSERT_MESSAGE("no manager available", m_manager);
     Strigi::IndexReader* reader = m_manager->indexReader();
     CPPUNIT_ASSERT_MESSAGE("reader creation failed", reader);
 }
 
 void
 IndexManagerTest::testIndexWriter() {
+    CPPUNIT_ASSERT_MESSAGE("no manager available", m_manager);
     Strigi::IndexWriter* writer = m_manager->indexWriter();
     CPPUNIT_ASSERT_MESSAGE("writer creation failed", writer);
 }
+void
+IndexManagerTest::testIndexManagerOnInvalidDirectory() {
+    // Creating an index manager on a non exitent directory should not cause
+    // major problems. Because in CLucene this can nevertheless lead to
+    // 'unknown errors' we have this test. It is there to verify that
+    // this code should not crash the program or throw an uncaught exception.
+    const char *dir = "i am a non-existent / directory";
+    Strigi::IndexManager* manager
+        = Strigi::IndexPluginLoader::createIndexManager(
+            m_backendname.c_str(), dir);
+    Strigi::IndexPluginLoader::deleteIndexManager(manager);
+}
